#The modbus_reference_tables are applicable to the "modbus" and "modbus_seametrics" drivers.
#The only difference between the two: "modbus_seametrics" takes two samples with 1 sec. wait time in between and discards the first.
#
# Coil	            Read-write	1 bit	    00001 – 09999 (traditional)  or  000001 – 065536 (full-range)
# Discrete input	Read-only	1 bit	    10001 – 19999 (traditional)  or  100001 – 165536 (full-range)
# Input register	Read-only	16 bits	    30001 – 39999 (traditional)  or  300001 – 365536 (full-range)
# Holding register	Read-write	16 bits	    40001 – 49999 (traditional)  or  400001 – 465536 (full-range)

#Define the table in the config.txt first, example:
#    modbus:
#    - id: modbus_1
#      protocol: RTU
#      register_print: True
#      sample_interval: "0 * * * *"
#      port: serial1
#      mode: RS485
#      rxbuf: 1024
#      baudrate: 9600
#      reference_table: modbus_table_1,modbus_table_2
#
#You can select a single or multiple tables.
#Dont forget to make the rxbuf size larger if a lot of registers are to be read
#
#Then create the table, example:
#    modbus_table_1 =    {  #0:    slave_addr, register_type, start_register, nr_of_registers
#                            0:    (0x01, register_type.INPUT_REGISTER, 0x00, 16),
#                           #x:    name, unit, tag, start_addr of chunk (zero is first), format, factor, offset
#                            1:    (("Modbus_1",    "UNSIGNED_SHORT",   "ModA" ), 0, formats.UNSIGNED_SHORT, 1, 0),
#                            2:    (("Modbus_2",    "SIGNED_SHORT",   "ModB" ), 1, formats.SIGNED_SHORT, 1, 0),
#                            3:    (("Modbus_3",    "SIGNED_INT",   "ModC" ), 2, formats.SIGNED_INT, 1, 0),
#                            4:    (("Modbus_4",    "FLOAT",   "ModD" ), 4, formats.FLOAT, 1, 0),
#                            5:    (("Modbus_5",    "DOUBLE",   "ModE" ), 12, formats.DOUBLE, 1, 0),
#                        }
#
#Use the configs "Omc048:" "sensor_data_print: True" to validate if all the formats are as desired (these are always printed as unsigned short).


class register_type:
    COIL                = 0
    DISCRETE_INPUT      = 1
    INPUT_REGISTER      = 3 
    HOLDING_REGISTER    = 4

class formats:
    UNSIGNED_SHORT      = 1   #WORD     (1 register of 16 bit)
    SIGNED_SHORT        = 2   #WORD     (1 register of 16 bit)
    UNSIGNED_INT        = 3   #DWORD    (2 registers of 16 bit)
    SIGNED_INT          = 4   #DWORD    (2 registers of 16 bit)
    FLOAT               = 5   #DWORD    (2 registers of 16 bit)
    FLOAT_INV           = 6   #DWORD    (2 registers of 16 bit) (registers swapped)
    DOUBLE              = 7   #QWORD    (4 registers of 16 bit)
    DOUBLE_INV          = 8   #QWORD    (4 registers of 16 bit) (registers swapped)

class  modbus_reference_table:
        
    #NOTE: COIL and DISCRETE_INPUT MUST BE A MULTIPLE OF 8 REGISTERS
    modbus_table_1 =    {           #0:    slave_addr, register_type, start_register, nr_of_registers
                            0:    (0x01, register_type.INPUT_REGISTER, 0x1, 50),
                            #x:    name, unit, tag, start_addr of chunk (zero is first), factor, offset
                            1:    (("UV",    "",   "UVC" ), 0, formats.FLOAT, 1, 0),
                            2:    (("Conductivity mS/cm",    "mS/cm",   "CONm" ), 2, formats.FLOAT, 1, 0),
                            3:    (("Temperature C",    "C",   "TMPC" ), 4, formats.FLOAT, 1, 0),
                }

    mosens =    {           #0:    slave_addr, register_type, start_register, nr_of_registers
                            0:    (0x80, register_type.INPUT_REGISTER, 0x100, 6),
                            #x:    name, unit, tag, start_addr of chunk (zero is first), factor, offset
                            1:    (("UV",    "",   "UVC" ), 0, formats.FLOAT, 1, 0),
                            2:    (("Conductivity mS/cm",    "mS/cm",   "CONm" ), 2, formats.FLOAT, 1, 0),
                            3:    (("Temperature C",    "C",   "TMPC" ), 4, formats.FLOAT, 1, 0),
                }

    TriOS_CT =  {           #0:    slave_addr, register_type, start_register, nr_of_registers
                            0:    (0xF0, register_type.HOLDING_REGISTER, 0x000, 7),
                            #x:    name, unit, tag, start_addr of chunk (zero is first), factor, offset
                            1:    (("Adres",    "",   "ADDR" ), 0, formats.UNSIGNED_SHORT, 1, 0),
                            2:    (("Baud",    "",   "BAUD" ), 1, formats.UNSIGNED_SHORT, 1, 0),
                            3:    (("Serial",    "",   "SERI" ), 2, formats.UNSIGNED_SHORT, 1, 0),
                            4:    (("Conductivity uS/cm",    "uS/cm",   "CONm" ), 3, formats.FLOAT, 1, 0),
                            5:    (("Temperature C",    "C",   "TMPC" ), 5, formats.FLOAT, 1, 0),
                }

    TriOS_NICO ={           #0:    slave_addr, register_type, start_register, nr_of_registers
                            0:    (0x01, register_type.HOLDING_REGISTER, 0x3E8, 14),
                            #x:    name, unit, tag, start_addr of chunk (zero is first), factor, offset
                            1:    (("N-NO3", "mg/l",  "N-NO3"), 0, formats.FLOAT, 1, 0),
                            2:    (("NO3", "mg/l", "NO3"), 2, formats.FLOAT, 1, 0),
                            3:    (("SQI", "", "SQI"), 4, formats.FLOAT, 1, 0),
                            4:    (("RefA", "", "RefA"), 6, formats.FLOAT, 1, 0),
                            5:    (("RefB", "", "RefB"), 8, formats.FLOAT, 1, 0),
                            6:    (("RefC", "", "RefC"), 10, formats.FLOAT, 1, 0),
                            7:    (("RefD", "", "RefD"), 12, formats.FLOAT, 1, 0),
                }

# Below for use with modbus_seametrics driver                
    PT12_1 =  {             #0:    slave_addr, register_type, start_register, nr_of_registers
                            0:    (0x01, register_type.HOLDING_REGISTER, 0x000, 6),
                            #x:    name, unit, tag, start_addr of chunk (zero is first), factor, offset
                            1:    (("Pressure PSI",    "PSI",   "PRES" ), 0, formats.FLOAT, 1, 0),
                            2:    (("Temperature C",    "C",   "TMPC" ), 2, formats.FLOAT, 1, 0),
                            3:    (("Power Supply V",    "V",   "VOLT" ), 4, formats.FLOAT, 1, 0),
                }
                
    PT12_2 =  {             #0:    slave_addr, register_type, start_register, nr_of_registers
                            0:    (0x02, register_type.HOLDING_REGISTER, 0x000, 14),
                            #x:    name, unit, tag, start_addr of chunk (zero is first), factor, offset
                            1:    (("Pressure PSI",    "PSI",   "PRES" ), 0, formats.FLOAT, 1, 0),
                            2:    (("Temperature C",    "C",   "TMPC" ), 2, formats.FLOAT, 1, 0),
                            3:    (("Power Supply V",    "V",   "VOLT" ), 4, formats.FLOAT, 1, 0),
                            4:    (("AVG Pressure PSI",    "PSI",   "AVPRES" ), 6, formats.FLOAT, 1, 0),
                            5:    (("MAX Pressure PSI",    "PSI",   "MAPRES" ), 8, formats.FLOAT, 1, 0),
                            6:    (("MIN Pressure PSI",    "PSI",   "MIPRES" ), 10, formats.FLOAT, 1, 0),
                            7:    (("AVG Temperature C",    "C",   "AVTMPC" ), 12, formats.FLOAT, 1, 0),
                }
                
    PT2X_1 =  {             #0:    slave_addr, register_type, start_register, nr_of_registers
                            0:    (0x01, register_type.HOLDING_REGISTER, 0xF480, 4),
                            #x:    name, unit, tag, start_addr of chunk (zero is first), factor, offset
                            1:    (("Temperature C",    "C",   "TMPC" ), 0, formats.FLOAT, 1, 0),
                            2:    (("Pressure PSI",    "PSI",   "PRES" ), 2, formats.FLOAT, 1, 0),
                }
 
    PT2X_2 =  {             #0:    slave_addr, register_type, start_register, nr_of_registers
                            0:    (0x02, register_type.HOLDING_REGISTER, 0xF480, 4),
                            #x:    name, unit, tag, start_addr of chunk (zero is first), factor, offset
                            1:    (("Temperature C",    "C",   "TMPC" ), 0, formats.FLOAT, 1, 0),
                            2:    (("Pressure PSI",    "PSI",   "PRES" ), 2, formats.FLOAT, 1, 0),
                }
                
    CT2X =  {               #0:    slave_addr, register_type, start_register, nr_of_registers
                            0:    (0x02, register_type.HOLDING_REGISTER, 0xF480, 12),
                            #x:    name, unit, tag, start_addr of chunk (zero is first), factor, offset
                            1:    (("Temperature C",         "C",       "TMPC" ), 0, formats.FLOAT, 1, 0),
                            2:    (("Conductivity uS/cm",    "uS/cm",   "CONu" ), 2, formats.FLOAT, 1, 0),
                            3:    (("nLF Cond uS/cm"    ,    "uS/cm",   "NCONu" ), 4, formats.FLOAT, 1, 0),
                            4:    (("Pressure PSI",          "PSI",     "PRES" ), 6, formats.FLOAT, 1, 0),
                            5:    (("Salinity PSU",          "PSU",     "SAL" ), 8, formats.FLOAT, 1, 0),
                            6:    (("TDS mg/L",              "mg/L",    "TDS"   ), 10, formats.FLOAT, 1, 0),
                }
                
# Below are RWS Specials
    RWS_Offshore =  {       #0:    slave_addr, register_type, start_register, nr_of_registers
                            0:    (0x01, register_type.HOLDING_REGISTER, 1000, 3),
                            #x:    name, unit, tag, start_addr of chunk (zero is first), factor, offset
                            1:    (("Temperature",    "C",   "TMP" ), 0, formats.FLOAT, 0.1, 0),
                            2:    (("MicroAmpere",    "uA",   "UAM" ), 2, formats.UNSIGNED_SHORT, 1, 0),
                            # 3:    (("Temperature_Channel",    "",   "TMC" ), 4, formats.FLOAT, 1, 0),
                }

    RWS_MoSens =    {       #0:    slave_addr, register_type, start_register, nr_of_registers
                            0:    (0x80, register_type.INPUT_REGISTER, 0x101, 4),
                            #x:    name, unit, tag, start_addr of chunk (zero is first), factor, offset
                            #1:    (("UV",    "",   "UVC" ), 0, formats.FLOAT, 1, 0),
                            1:    (("Conductivity",    "mS/cm",   "CONm" ), 0, formats.FLOAT, 1, 0),
                            2:    (("Temperature",    "C",   "TEMP" ), 2, formats.FLOAT, 1, 0),
                }
 